/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/* 
 * imsettings-stat.c
 * Copyright (C) 2011 Red Hat, Inc. All rights reserved.
 * 
 * Authors:
 *   Akira TAGOH  <tagoh@redhat.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <glib.h>
#include <glib/gi18n.h>
#include <gio/gio.h>
#include <X11/Xlib.h>
#include "imsettings.h"
#include "imsettings-client.h"

/*< private >*/
static void
_log(const gchar *message,
     gboolean     result,
     const gchar *reason)
{
	g_print("%s: ", message);
	g_print("%s\n", result ? "true" : "false");
	if (!result) {
		g_print("  reason: %s\n", reason ? _(reason) : "unknown");
	}
}

static gboolean
_check_dbus(gboolean verbose)
{
	GDBusConnection *connection;
	gboolean retval = TRUE;
	GError *error = NULL;

	connection = g_bus_get_sync(G_BUS_TYPE_SESSION,
				    NULL,
				    &error);
	if (error)
		retval = FALSE;

	if (connection)
		g_object_unref(connection);

	if (verbose) {
		_log("DBus availability",
		     retval,
		     error ? error->message : NULL);
	}
	if (error)
		g_error_free(error);

	return retval;
}

static gboolean
_check_xsettings(gboolean verbose)
{
	Display *dpy = XOpenDisplay(NULL);
	char buffer[256];
	Atom selection_atom;
	GError *error = NULL;
	gboolean retval = TRUE;

	if (!dpy) {
		g_set_error(&error, 0, 0, N_("Unable to open X display"));
		retval = FALSE;
		goto bail;
	}
	sprintf(buffer, "_XSETTINGS_S%d", DefaultScreen(dpy));
	selection_atom = XInternAtom(dpy, buffer, False);
	if (!XGetSelectionOwner(dpy, selection_atom)) {
		g_set_error(&error, 0, 0, N_("XSETTINGS manager isn't running"));
		retval = FALSE;
		goto bail;
	}
  bail:
	if (dpy)
		XCloseDisplay(dpy);
	if (verbose) {
		_log("XSETTINGS availability",
		     retval,
		     error ? error->message : NULL);
	}
	if (error)
		g_error_free(error);

	return retval;
}

static gboolean
_check_modules(gboolean verbose)
{
	GString *current_settings = g_string_new(NULL);
	gchar *locale = setlocale(LC_CTYPE, NULL);
	IMSettingsClient *client = imsettings_client_new(locale);
	GVariant *v = NULL;
	GVariantIter *iter;
	const gchar *key, *val, *last_val = NULL;
	gboolean no_cmp = TRUE, retval = TRUE;
	GError *error = NULL;

	if (!client) {
		g_set_error(&error, 0, 0,
			    N_("Unable to create a client instance."));
		retval = FALSE;
		goto bail;
	}
	if (imsettings_client_get_version(client, NULL, &error) != IMSETTINGS_SETTINGS_API_VERSION) {
		retval = FALSE;
		if (!error) {
			g_set_error(&error, 0, 0,
				    N_("imsettings version mismatch"));
		}
		goto bail;
	}
	v = imsettings_client_get_module_settings(client, NULL, &error);
	if (error) {
		retval = FALSE;
		goto bail;
	}
	g_variant_get(v, "a{ss}", &iter);
	while (g_variant_iter_next(iter, "{&s&s}", &key, &val)) {
		if (no_cmp) {
			last_val = val;
			no_cmp = FALSE;
		} else {
			if (retval && g_strcmp0(last_val, val) != 0) {
				retval = FALSE;
				g_set_error(&error, 0, 0,
					    N_("Please see .imsettings.log for more details"));
			}
			last_val = val;
		}
		g_string_append_printf(current_settings, "    %s:\t%s\n", key, val);
	}
	g_variant_iter_free(iter);
	g_variant_unref(v);
  bail:
	if (client)
		g_object_unref(client);
	if (verbose) {
		_log("Current settings in modules", retval,
		     error ? error->message : NULL);
		g_print("  result:\n%s", current_settings->str);
	}
	if (error)
		g_error_free(error);
	g_string_free(current_settings, TRUE);

	return retval;
}

/*< public >*/

int
main(int    argc,
     char **argv)
{
	gboolean arg_detail = FALSE;
	GOptionContext *ctx = g_option_context_new(NULL);
	GOptionEntry entries[] = {
		/* For translators: this is a translation for the command-line option. */
		{"detail", 'd', 0, G_OPTION_ARG_NONE, &arg_detail, N_("Output the detail information for the result"), NULL},
		{NULL, 0, 0, 0, NULL, NULL, NULL}
	};
	GError *error = NULL;
	int retval = 0;

#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, IMSETTINGS_LOCALEDIR);
#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
#endif /* HAVE_BIND_TEXTDOMAIN_CODESET */
	textdomain (GETTEXT_PACKAGE);
#endif /* ENABLE_NLS */

	setlocale(LC_ALL, "");

	g_type_init();

	g_option_context_add_main_entries(ctx, entries, GETTEXT_PACKAGE);
	if (!g_option_context_parse(ctx, &argc, &argv, &error)) {
		if (error != NULL) {
			g_printerr("%s\n", error->message);
		} else {
			g_warning(_("Unknown error in parsing the command lines."));
		}
		return 1;
	}

	/* check if DBus is running */
	if (!_check_dbus(arg_detail)) {
		retval = 1;
		goto end;
	}

	/* check if XSETTINGS manager is running */
	if (!_check_xsettings(arg_detail)) {
		retval = 1;
		goto end;
	}

	/* check current settings */
	if (!_check_modules(arg_detail)) {
		retval = 1;
		goto end;
	}

  end:

	return retval;
}
